DEFINT A-Z

start:

REM INPUT "prn"; PRNID     'in case you want to edit out the REM
FOR PRNID = 1 TO 37        'but FOR/NEXT loop ended up being more useful

' This code makes a HEX table of the 1023 bit C/A spreading code for all
' possible GPS satellites and pseudolites.
'
'           T.A. Clark              May 1993
'
' The following is a table of the two register tap addresses for the G2
' register needed to generate the "legal" GPS C/A "Gold Codes". The G2
' register has 10 taps numbered (what else?) 1-10 with #1 being nearest
' to the feedback input and #10 being ten chips (10.23 usec) delayed.
'
' The G1 register is defined as 1 + x^3 + x^10
' and the G2 register is 1 + x^2 + x^3 + x^6 + x^8 + x^9 + x^10
'
' The C/A code is then the modulo-2 sum of G1(10) + G2(T1) + G2(T2)
'
' The G1 and G2 registers are initialized to all ones (1023) at the
' satellites UTC/GPS time tick epoch and become all ones every 1023
' clock cycles ( 1.023 MHz/1023 counts = 1 msec ) thereafter.

   SELECT CASE PRNID
        CASE 1:  T1 = 2: T2 = 6
        CASE 2:  T1 = 3: T2 = 7
        CASE 3:  T1 = 4: T2 = 8
        CASE 4:  T1 = 5: T2 = 9
        CASE 5:  T1 = 1: T2 = 9
        CASE 6:  T1 = 2: T2 = 10
        CASE 7:  T1 = 1: T2 = 8
        CASE 8:  T1 = 2: T2 = 9
        CASE 9:  T1 = 3: T2 = 10
        CASE 10: T1 = 2: T2 = 3
        CASE 11: T1 = 3: T2 = 4
        CASE 12: T1 = 5: T2 = 6
        CASE 13: T1 = 6: T2 = 7
        CASE 14: T1 = 7: T2 = 8
        CASE 15: T1 = 8: T2 = 9
        CASE 16: T1 = 9: T2 = 10
        CASE 17: T1 = 1: T2 = 4
        CASE 18: T1 = 2: T2 = 5
        CASE 19: T1 = 3: T2 = 6
        CASE 20: T1 = 4: T2 = 7
        CASE 21: T1 = 5: T2 = 8
        CASE 22: T1 = 6: T2 = 9
        CASE 23: T1 = 1: T2 = 3
        CASE 24: T1 = 4: T2 = 6
        CASE 25: T1 = 5: T2 = 7
        CASE 26: T1 = 6: T2 = 8
        CASE 27: T1 = 7: T2 = 9
        CASE 28: T1 = 8: T2 = 10
        CASE 29: T1 = 1: T2 = 6
        CASE 30: T1 = 2: T2 = 7
        CASE 31: T1 = 3: T2 = 8
        CASE 32: T1 = 4: T2 = 9
        CASE 33: T1 = 5: T2 = 10
        CASE 34: T1 = 4: T2 = 10
        CASE 35: T1 = 1: T2 = 7
        CASE 36: T1 = 2: T2 = 8
        CASE 37: T1 = 4: T2 = 10
        CASE ELSE: PRINT "INVALID PRN": STOP
        END SELECT

G1 = 1023: G2 = 1023:  'Initialize G1 and G2 registers to all ones
C$ = "": chk10 = 0:    'C$=Hex code table for output, chk10 is 1st 10 bits
bias = 0!: count = 0!
Length = 1024

FOR i = 1 TO Length STEP 4
  CT = 0:     'CT is the 4-step CODE nibble

  FOR j = i TO i + 3:   'Count in 4's to fill Hex characters

   'Test to see if we are done, even if we are in this little loop:
     IF j <= Length THEN

      'Beginning of the next cycle when all ones are in G1 and G2
      '   IF G1 = 1023 THEN PRINT "G1 Register = 1111111111 at count = "; j
      '   IF G2 = 1023 THEN PRINT "G2 Register = 1111111111 at count = "; j

         G1k = G1: G2k = G2:   'Temporary values to extract register taps

         FOR k = 1 TO 10:      '10 taps in G1 and G2 Shift Registers
           G1tap(k) = G1k MOD 2: G2tap(k) = G2k MOD 2
           G1k = INT(G1k / 2): G2k = INT(G2k / 2)
         NEXT k

       'G1 register equation 1 + x^3 + x^10
           G1fb = G1tap(3) + G1tap(10)
       'G2 register equation 1 + x^2 + x^3 + x^6 + x^8 + x^9 + x^10:
           G2fb = G2tap(2) + G2tap(3) + G2tap(6) + G2tap(8) + G2tap(9) + G2tap(10)
       'And take Modulo-2 (one bit) feedback value:
           G1fb = G1fb MOD 2: G2fb = G2fb MOD 2
       'Finally, generate the next code state with a shift:
           G1 = (2 * G1 + G1fb) MOD 1024: G2 = (2 * G2 + G2fb) MOD 1024


       'The C/A code is G1 tap#10 plus 2 taps from G2 determined by PRN:
           code = (G1tap(10) + G2tap(T1) + G2tap(T2)) MOD 2
           CT = 2 * CT + code
     
       'How close is this to a 50/50 zero/one distribution?
           bias = bias + (1! * code): count = count + 1!

       'For convenience in verifying GPS ICD, which tabulates the 1st 10 bits:
           IF j <= 10 THEN chk10 = chk10 * 2 + code
           IF j = 10 THEN PRINT "--------------- PRN = "; PRNID; " --------------"
           IF j = 10 THEN PRINT "1st 10 bits of code (octal) = "; OCT$(chk10)
  
       'and Just in case Length doesn't fill out the block:
    ELSE
       CT = 2 * CT
    END IF
  NEXT j

  'Write the last 4 bits into a Hex character:
    C$ = C$ + HEX$(CT)

NEXT i


' Finally print out the C/A code in HEX format. These lines could be modified
' to produce the bits to fill a code generator ROM

PRINT : PRINT "       C/A CODE FOR GPS PRN = "; PRNID
FOR i = 1 TO LEN(C$) STEP 64
   PRINT i * 4 - 3; ":"; TAB(9);
      FOR j = i TO i + 63 STEP 8
         PRINT MID$(C$, j, 8); " ";
      NEXT j
   PRINT
NEXT i


'The following is some summary stuff I wanted to see:
PRINT : PRINT "    PRN = "; PRNID;

'The following it to verify code against ICD-200, which lists the first ten
'bits of the code:
PRINT "       1st 10 bits for check : "; OCT$(chk10); " Octal"

'The following is because I was interested in testing to see that each code
'was in fact a zero-mean process:
'PRINT "In this sample there were "; bias; " ones our of "; count; " samples. "
'PRINT "The excess is "; bias - (count / 2!); " bits."
'PRINT " This implies a bias of "; (bias * 2 / count) - 1; " in the +/- 1 correlations."
'PRINT "------------------------------------------------------------": PRINT

'Finally, close the PRNID FOR/NEXT loop and quit:
NEXT PRNID
END

